import { IDataObject, INode, JsonObject } from '.';
export declare abstract class ExecutionBaseError extends Error {
    description: string | null | undefined;
    cause: Error | JsonObject;
    timestamp: number;
    context: IDataObject;
    constructor(error: Error | ExecutionBaseError | JsonObject);
}
declare abstract class NodeError extends ExecutionBaseError {
    node: INode;
    constructor(node: INode, error: Error | JsonObject);
    protected findProperty(error: JsonObject, potentialKeys: string[], traversalKeys?: string[]): string | null;
    protected isTraversableObject(value: any): value is JsonObject;
    protected removeCircularRefs(obj: JsonObject, seen?: Set<unknown>): void;
}
export declare class NodeOperationError extends NodeError {
    constructor(node: INode, error: Error | string, options?: {
        description?: string;
        runIndex?: number;
        itemIndex?: number;
    });
}
export declare class NodeApiError extends NodeError {
    httpCode: string | null;
    constructor(node: INode, error: JsonObject, { message, description, httpCode, parseXml, runIndex, itemIndex, }?: {
        message?: string;
        description?: string;
        httpCode?: string;
        parseXml?: boolean;
        runIndex?: number;
        itemIndex?: number;
    });
    private setDescriptionFromXml;
    private setMessage;
}
export {};
